/**
 * @file arduino_user_event_example.ino
 * @author Shinichiro Nakamura
 * @copyright
 * ===============================================================
 * A tiny MML parser (Version 0.5.0)
 * ===============================================================
 * Copyright (c) 2014-2015 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @note Please change MML_CONFIG_USE_AVR_PROGMEM in mml.h to (1) for this sample codes.
 */
#include <mml.h>
#include <Servo.h>
#include "chord.h"
#include "song.h"

CHORD chord;
MML mml;
MML_OPTION mml_opt;
Servo servo;

static void mml_callback(MML_INFO *p, void *extobj)
{
  switch (p->type) {
    case MML_TYPE_TEMPO:
      {
        MML_ARGS_TEMPO *args = &(p->args.tempo);
        chord_init(&chord, args->value, mml_opt.bticks);
      }
      break;
    case MML_TYPE_NOTE:
      {
        MML_ARGS_NOTE *args = &(p->args.note);
        chord_note(&chord, args->number, args->ticks);
      }
      break;
    case MML_TYPE_REST:
      {
        MML_ARGS_REST *args = &(p->args.rest);
        chord_rest(&chord, args->ticks);
      }
      break;
    case MML_TYPE_USER_EVENT:
      {
        MML_ARGS_USER_EVENT *args = &(p->args.user_event);
        if (strcmp(args->name, "EVT1") == 0) { servo.write(  0); }
        if (strcmp(args->name, "EVT2") == 0) { servo.write( 45); }
        if (strcmp(args->name, "EVT3") == 0) { servo.write( 90); }
        if (strcmp(args->name, "EVT4") == 0) { servo.write(135); }
        if (strcmp(args->name, "EVT5") == 0) { servo.write(180); }
        {
          char *p = args->name;
          while (*p) {
            Serial.write(*p);
            p++;
          }
          Serial.write('\n');
        }
      }
      break;
  }
}

void setup()
{
  Serial.begin(9600);
  servo.attach(9);

  /*
   * Initialize the MML module.
   */
  mml_init(&mml, mml_callback, 0);
  MML_OPTION_INITIALIZER_DEFAULT(&mml_opt);

  /*
   * Initialize the chord module.
   */
  int tempo_default = 120;
  chord_init(&chord, tempo_default, mml_opt.bticks);
}

void loop()
{
  const char *song_table[] = {
    song_test1,
    song_test2,
  };
  int i;

  for (i = 0; i < sizeof(song_table) / sizeof(song_table[0]); i++) {
    /*
     * Setup the MML module.
     */
    mml_setup(&mml, &mml_opt, (char *)song_table[i]);

    /*
     * Fetch the MML text command.
     */
    while (mml_fetch(&mml) == MML_RESULT_OK) {
    }
  }
}

